import { WarningModal } from './warnings'


/**
 * Defines the available `views` that can be used to set up routes.
 * Each `Template` has different configuration options
 */
export type Template = 
  | 'markdown'
  | 'project-list'
  | 'gallery-list'

/**
 * Defines the shared options for a route
 */
export type SharedRouteDefinition = {
  id: string
  scriptUrl?: string
  stylesheetUrls: string[]
  template: Template
  title: string
  warning: boolean | WarningModal
}

/**
 * Defines a content-based route
 */
export type ContentfulRouteDefintion = SharedRouteDefinition & {
  content: string
}

/**
 * Defines a config-based route
 */
export type ConfigfulRouteDefinition = SharedRouteDefinition & {
  config: string
}

/**
 * Defines the config for a route using the `markdown` {@link Template}
 */
export type MarkdownDefinition = ContentfulRouteDefintion & {
  template: 'markdown'
}

/**
 * Defines the config for a route using the `project-list` {@link Template}
 */
export type ProjectListDefinition = ConfigfulRouteDefinition & {
  template: 'project-list'
  view: {
    stylesheetUrls: string[]
  }
}

/**
 * Defines the config for a route using the `gallery-list` {@link Template}
 */
export type GalleryListDefinition = ConfigfulRouteDefinition & {
  template: 'gallery-list'
  view: {
    stylesheetUrls: string[]
  }
}

/**
 * Defines all available route definitions
 */
export type RouteDefinition =
  | MarkdownDefinition
  | ProjectListDefinition
  | GalleryListDefinition

/**
 * Defines the collection of routes the app uses
 */
export type RouteCollection = { [key: string]: RouteDefinition }
